@@ -15,34 +15,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: ephy-bookmarks-editor.c 7108 2007-07-01 14:04:43Z jhaitsma $
  */
 
 #include "config.h"
 
-#include <gtk/gtktable.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtkscrolledwindow.h>
-#include <gtk/gtkhpaned.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkactiongroup.h>
-#include <gtk/gtkcombobox.h>
-#include <gtk/gtkuimanager.h>
-#include <gtk/gtktoggleaction.h>
-#include <gtk/gtktreeselection.h>
-#include <gtk/gtktreemodelsort.h>
-#include <gtk/gtkcelllayout.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtkradioaction.h>
-#include <gtk/gtkclipboard.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtktreemodelsort.h>
-#include <gtk/gtkmessagedialog.h>
+#include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <glib/gi18n.h>
-#include <libgnomeui/gnome-stock-icons.h>
 #include <string.h>
 
 #include "ephy-bookmarks-editor.h"
@@ -55,6 +34,7 @@
 #include "ephy-node-view.h"
 #include "ephy-window.h"
 #include "ephy-dnd.h"
+#include "ephy-settings.h"
 #include "ephy-shell.h"
 #include "ephy-session.h"
 #include "ephy-file-helpers.h"
@@ -66,7 +46,6 @@
 #include "ephy-stock-icons.h"
 #include "ephy-search-entry.h"
 #include "ephy-favicon-cache.h"
-#include "eel-gconf-extensions.h"
 #include "ephy-debug.h"
 #include "egg-toolbars-model.h"
 #include "ephy-prefs.h"
@@ -147,8 +126,6 @@ static void cmd_help_contents		  (GtkAction *action,
 
 #define EPHY_BOOKMARKS_EDITOR_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_BOOKMARKS_EDITOR, EphyBookmarksEditorPrivate))
 
-#define CONF_BOOKMARKS_VIEW_DETAILS "/apps/epiphany/dialogs/bookmarks_view_details"
-
 #define RESERVED_STRING N_("Remove from this topic")
 
 struct _EphyBookmarksEditorPrivate
@@ -174,8 +151,6 @@ enum
 	PROP_BOOKMARKS
 };
 
-static GObjectClass *parent_class = NULL;
-
 static const GtkActionEntry ephy_bookmark_popup_entries [] = {
 	/* Toplevel */
 	{ "File", NULL, N_("_File") },
@@ -239,24 +214,15 @@ static const GtkToggleActionEntry ephy_bookmark_toggle_entries [] = {
 	{ "ShowOnToolbar", NULL, N_("_Show on Toolbar"), NULL,
 	  N_("Show the selected bookmark on a toolbar"), 
 	  G_CALLBACK (cmd_toolbar), FALSE },
-};
-
-enum
-{
-	VIEW_TITLE,
-	VIEW_TITLE_AND_ADDRESS
-};
-
-static const GtkRadioActionEntry ephy_bookmark_radio_entries [] =
-{
 	/* View Menu */
 	{ "ViewTitle", NULL, N_("_Title"), NULL,
-	  N_("Show only the title column"), VIEW_TITLE },
-	{ "ViewTitleAddress", NULL, N_("T_itle and Address"), NULL,
-	  N_("Show both the title and address columns"),
-	  VIEW_TITLE_AND_ADDRESS } 
+	  N_("Show the title column"), NULL, TRUE },
+	{ "ViewAddress", NULL, N_("Address"), NULL,
+	  N_("Show the address column"), NULL, FALSE }
 };
 
+G_DEFINE_TYPE (EphyBookmarksEditor, ephy_bookmarks_editor, GTK_TYPE_WINDOW)
+
 static void
 entry_selection_changed_cb (GtkWidget *widget, GParamSpec *pspec, EphyBookmarksEditor *editor)
 {
@@ -353,15 +319,15 @@ cmd_open_bookmarks_in_tabs (GtkAction *action,
 	for (l = selection; l; l = l->next)
 	{
 		EphyNode *node = l->data;
-		EphyTab *new_tab;
+		EphyEmbed *new_embed;
 		const char *location;
 
 		location = ephy_node_get_property_string (node,
 						EPHY_NODE_BMK_PROP_LOCATION);
 
-		new_tab = ephy_shell_new_tab (ephy_shell, window, NULL, location,
-					      EPHY_NEW_TAB_OPEN_PAGE |
-					      EPHY_NEW_TAB_IN_EXISTING_WINDOW);
+		new_embed = ephy_shell_new_tab (ephy_shell, window, NULL, location,
+				 	        EPHY_NEW_TAB_OPEN_PAGE |
+					        EPHY_NEW_TAB_IN_EXISTING_WINDOW);
 		/* if there was no target window, a new one was opened. Get it
 		 * from the new tab so we open the remaining links in the
 		 * same window. See bug 138343.
@@ -369,7 +335,7 @@ cmd_open_bookmarks_in_tabs (GtkAction *action,
 		if (window == NULL)
 		{
 			window = EPHY_WINDOW
-				(gtk_widget_get_toplevel (GTK_WIDGET (new_tab)));
+				(gtk_widget_get_toplevel (GTK_WIDGET (new_embed)));
 		}
 	}
 
@@ -423,7 +389,8 @@ delete_topic_dialog_construct (GtkWindow *parent,
 	gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Delete Topic"), GTK_RESPONSE_ACCEPT);
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CANCEL);
 
-	gtk_window_group_add_window (GTK_WINDOW (parent)->group, GTK_WINDOW (dialog));
+	gtk_window_group_add_window (gtk_window_get_group (GTK_WINDOW (parent)),
+				     GTK_WINDOW (dialog));
 
 	return dialog;
 }
@@ -706,7 +673,7 @@ import_bookmarks (EphyBookmarksEditor *editor,
 			   "unsupported type."),
 			 basename);
 
-		gtk_window_group_add_window (GTK_WINDOW (editor)->group,
+		gtk_window_group_add_window (gtk_window_get_group (GTK_WINDOW (editor)),
 					     GTK_WINDOW (dialog));
 
 		gtk_dialog_run (GTK_DIALOG (dialog));
@@ -922,13 +889,13 @@ cmd_bookmarks_export (GtkAction *action,
 	/* Make a format selection combo & label */
 	label = gtk_label_new_with_mnemonic (_("File f_ormat:"));
 
-	combo = gtk_combo_box_new_text ();
+	combo = gtk_combo_box_text_new ();
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), combo);
 
 	for (format = 0; format < G_N_ELEMENTS (export_formats); ++format)
 	{
-		gtk_combo_box_append_text (GTK_COMBO_BOX (combo),
-					   _(export_formats[format].name));
+		gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo),
+					        _(export_formats[format].name));
 	}
 
 	g_object_set_data (G_OBJECT (dialog), "format-combo", combo);
@@ -943,7 +910,8 @@ cmd_bookmarks_export (GtkAction *action,
 	
 	gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (dialog), hbox);
 
-	gtk_window_group_add_window (GTK_WINDOW (editor)->group, GTK_WINDOW (dialog));
+	gtk_window_group_add_window (gtk_window_get_group (GTK_WINDOW (editor)),
+				     GTK_WINDOW (dialog));
 
 	g_signal_connect (dialog, "response",
 			  G_CALLBACK (export_dialog_response_cb), editor);
@@ -955,6 +923,7 @@ cmd_bookmarks_import (GtkAction *action,
 		      EphyBookmarksEditor *editor)
 {
 	GtkWidget *dialog;
+	GtkWidget *content_area;
 	GtkWidget *label;
 	GtkWidget *vbox;
 	GtkWidget *combo;
@@ -966,22 +935,23 @@ cmd_bookmarks_import (GtkAction *action,
 
 	dialog = gtk_dialog_new_with_buttons (_("Import Bookmarks"),
 					      GTK_WINDOW (editor),
-					      GTK_DIALOG_DESTROY_WITH_PARENT |
-					      GTK_DIALOG_NO_SEPARATOR,
+					      GTK_DIALOG_DESTROY_WITH_PARENT,
 					      GTK_STOCK_CANCEL,
 					      GTK_RESPONSE_CANCEL,
 					      _("I_mport"),
 					      GTK_RESPONSE_OK,
 					      NULL);
+	content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
+
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
 	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
-	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 2);
+	gtk_box_set_spacing (GTK_BOX (content_area), 2);
 
 	vbox = gtk_vbox_new (FALSE, 6);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
 	gtk_widget_show (vbox);
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), vbox,
+	gtk_box_pack_start (GTK_BOX (content_area), vbox,
 			    TRUE, TRUE, 0);
 
 	label = gtk_label_new (_("Import bookmarks from:"));
@@ -1130,81 +1100,7 @@ static void
 cmd_help_contents (GtkAction *action,
 		   EphyBookmarksEditor *editor)
 {
-	ephy_gui_help (GTK_WINDOW (editor), 
-		       "epiphany", 
-		       "ephy-managing-bookmarks");
-}
-
-static void
-set_columns_visibility (EphyBookmarksEditor *editor, int value)
-{
-	switch (value)
-	{
-		case VIEW_TITLE:
-			gtk_tree_view_column_set_visible (editor->priv->title_col, TRUE);
-			gtk_tree_view_column_set_visible (editor->priv->address_col, FALSE);
-			break;
-		case VIEW_TITLE_AND_ADDRESS:
-			gtk_tree_view_column_set_visible (editor->priv->title_col, TRUE);
-			gtk_tree_view_column_set_visible (editor->priv->address_col, TRUE);
-			break;
-	}
-}
-
-static void
-cmd_view_columns (GtkAction *action,
-		  GtkRadioAction *current,
-		  EphyBookmarksEditor *editor)
-{
-	int value;
-	GSList *svalues = NULL;
-
-	g_return_if_fail (EPHY_IS_BOOKMARKS_EDITOR (editor));
-
-	value = gtk_radio_action_get_current_value (current);
-	set_columns_visibility (editor, value);
-
-	switch (value)
-	{
-		case VIEW_TITLE:
-			svalues = g_slist_append (svalues, (gpointer)"title");
-			break;
-		case VIEW_TITLE_AND_ADDRESS:
-			svalues = g_slist_append (svalues, (gpointer)"title");
-			svalues = g_slist_append (svalues, (gpointer)"address");
-			break;
-	}
-
-	eel_gconf_set_string_list (CONF_BOOKMARKS_VIEW_DETAILS, svalues);
-	g_slist_free (svalues);
-}
-
-GType
-ephy_bookmarks_editor_get_type (void)
-{
-	static GType type = 0;
-
-	if (G_UNLIKELY (type == 0))
-	{
-		const GTypeInfo our_info =
-		{
-			sizeof (EphyBookmarksEditorClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) ephy_bookmarks_editor_class_init,
-			NULL,
-			NULL,
-			sizeof (EphyBookmarksEditor),
-			0,
-			(GInstanceInitFunc) ephy_bookmarks_editor_init
-		};
-
-		type = g_type_register_static (GTK_TYPE_WINDOW,
-					       "EphyBookmarksEditor",
-					       &our_info, 0);
-	}
-
-	return type;
+	ephy_gui_help (GTK_WIDGET (editor), "ephy-managing-bookmarks");
 }
 
 static void
@@ -1214,7 +1110,7 @@ ephy_bookmarks_editor_show (GtkWidget *widget)
 
 	gtk_widget_grab_focus (editor->priv->search_entry);
 
-	GTK_WIDGET_CLASS (parent_class)->show (widget);
+	GTK_WIDGET_CLASS (ephy_bookmarks_editor_parent_class)->show (widget);
 }
 
 static void
@@ -1223,8 +1119,6 @@ ephy_bookmarks_editor_class_init (EphyBookmarksEditorClass *klass)
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	object_class->finalize = ephy_bookmarks_editor_finalize;
 	object_class->dispose  = ephy_bookmarks_editor_dispose;
 
@@ -1262,7 +1156,7 @@ ephy_bookmarks_editor_finalize (GObject *object)
 			 (gpointer *)window);
 	}
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (ephy_bookmarks_editor_parent_class)->finalize (object);
 }
 
 static void
@@ -1280,6 +1174,25 @@ ephy_bookmarks_editor_node_activated_cb (GtkWidget *view,
 			    EPHY_NEW_TAB_OPEN_PAGE);
 }
 
+static void
+ephy_bookmarks_editor_node_middle_clicked_cb (GtkWidget *view,
+					      EphyNode *node,
+					      EphyBookmarksEditor *editor)
+{
+	EphyWindow *window;
+	const char *location;
+
+	window = EPHY_WINDOW (get_target_window (editor));
+
+	location = ephy_node_get_property_string
+		(node, EPHY_NODE_BMK_PROP_LOCATION);
+	g_return_if_fail (location != NULL);
+
+	ephy_shell_new_tab (ephy_shell, window, NULL, location,
+			    EPHY_NEW_TAB_OPEN_PAGE |
+			    EPHY_NEW_TAB_IN_EXISTING_WINDOW);
+}
+
 static void
 ephy_bookmarks_editor_update_menu (EphyBookmarksEditor *editor)
 {
@@ -1498,7 +1411,7 @@ ephy_bookmarks_editor_dispose (GObject *object)
 		if (selection == NULL || selection->data == NULL)
 		{
 			editor->priv->key_view = NULL;
-			G_OBJECT_CLASS (parent_class)->dispose (object);
+			G_OBJECT_CLASS (ephy_bookmarks_editor_parent_class)->dispose (object);
 			return;
 		}
 
@@ -1507,7 +1420,7 @@ ephy_bookmarks_editor_dispose (GObject *object)
 		editor->priv->key_view = NULL;
 	}
 
-	G_OBJECT_CLASS (parent_class)->dispose (object);
+	G_OBJECT_CLASS (ephy_bookmarks_editor_parent_class)->dispose (object);
 }
 
 static void
@@ -1530,7 +1443,7 @@ key_pressed_cb (EphyNodeView *view,
 	EphyBookmarksEditorPrivate *priv = editor->priv;
 	GtkAction *action;
 
-	if (event->keyval == GDK_Delete || event->keyval == GDK_KP_Delete)
+	if (event->keyval == GDK_KEY_Delete || event->keyval == GDK_KEY_KP_Delete)
 	{
 		action = gtk_action_group_get_action (priv->action_group, "Delete");
 		if (gtk_action_get_sensitive (action))
@@ -1597,6 +1510,11 @@ search_entry_search_cb (GtkWidget *entry, const char *search_text, EphyBookmarks
 									  EPHY_NODE_BMK_PROP_TITLE,
 									  search_text),
 					 0);
+	ephy_node_filter_add_expression (editor->priv->bookmarks_filter,
+					 ephy_node_filter_expression_new (EPHY_NODE_FILTER_EXPRESSION_STRING_PROP_CONTAINS,
+									  EPHY_NODE_BMK_PROP_LOCATION,
+									  search_text),
+					 0);
 	ephy_node_filter_add_expression (editor->priv->bookmarks_filter,
 					 ephy_node_filter_expression_new (EPHY_NODE_FILTER_EXPRESSION_STRING_PROP_CONTAINS,
 									  EPHY_NODE_BMK_PROP_KEYWORDS,
@@ -1619,12 +1537,14 @@ build_search_box (EphyBookmarksEditor *editor)
 
 	entry = ephy_search_entry_new ();
 	editor->priv->search_entry = entry;
-	gtk_widget_show (entry);
+	
 	g_signal_connect (G_OBJECT (entry), "search",
 			  G_CALLBACK (search_entry_search_cb),
 			  editor);
 	add_entry_monitor (editor, entry);
 	add_focus_monitor (editor, entry);
+	
+	gtk_widget_show_all (entry);
 
 	label = gtk_label_new (NULL);
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
@@ -1722,31 +1642,6 @@ provide_keyword_uri (EphyNode *node, GValue *value, gpointer data)
 	g_free (uri);
 }
 
-static int
-get_details_value (EphyBookmarksEditor *editor)
-{
-	int value;
-	GSList *svalues;
-
-	svalues = eel_gconf_get_string_list (CONF_BOOKMARKS_VIEW_DETAILS);
-
-	if (svalues &&
-	    g_slist_find_custom (svalues, "title", (GCompareFunc)strcmp) &&
-	    g_slist_find_custom (svalues, "address", (GCompareFunc)strcmp))
-	{
-		value = VIEW_TITLE_AND_ADDRESS;
-	}
-	else
-	{
-		value = VIEW_TITLE;
-	}
-
-	g_slist_foreach (svalues, (GFunc) g_free, NULL);
-	g_slist_free (svalues);
-
-	return value;
-}
-
 static void
 ephy_bookmarks_editor_construct (EphyBookmarksEditor *editor)
 {
@@ -1758,7 +1653,7 @@ ephy_bookmarks_editor_construct (EphyBookmarksEditor *editor)
 	GtkUIManager *ui_merge;
 	GtkActionGroup *action_group;
 	GtkAction *action;
-	int col_id, url_col_id, title_col_id, details_value;
+	int col_id, url_col_id, title_col_id;
 
 	ephy_gui_ensure_window_group (GTK_WINDOW (editor));
 
@@ -1781,13 +1676,6 @@ ephy_bookmarks_editor_construct (EphyBookmarksEditor *editor)
 	gtk_action_group_add_toggle_actions (action_group, ephy_bookmark_toggle_entries,
 					     G_N_ELEMENTS (ephy_bookmark_toggle_entries), editor);
 
-	details_value = get_details_value (editor);
-	gtk_action_group_add_radio_actions (action_group,
-					    ephy_bookmark_radio_entries,
-					    G_N_ELEMENTS (ephy_bookmark_radio_entries),
-					    details_value,
-					    G_CALLBACK (cmd_view_columns), 
-					    editor);
 	gtk_ui_manager_insert_action_group (ui_merge,
 					    action_group, 0);
 	gtk_ui_manager_add_ui_from_file (ui_merge,
@@ -1899,12 +1787,22 @@ ephy_bookmarks_editor_construct (EphyBookmarksEditor *editor)
 				  (EPHY_NODE_VIEW (bm_view), _("Title"),
 				   G_TYPE_STRING, EPHY_NODE_BMK_PROP_TITLE,
 				   EPHY_NODE_VIEW_EDITABLE |
-				   EPHY_NODE_VIEW_SEARCHABLE,
+				   EPHY_NODE_VIEW_SEARCHABLE |
+				   EPHY_NODE_VIEW_SORTABLE |
+				   EPHY_NODE_VIEW_ELLIPSIZED,
 				   provide_favicon, &(editor->priv->title_col));
+	gtk_tree_view_column_set_min_width (editor->priv->title_col, 300);
+	gtk_tree_view_column_set_resizable (editor->priv->title_col, TRUE);
+
 	url_col_id = ephy_node_view_add_column
 				  (EPHY_NODE_VIEW (bm_view), _("Address"),
 				   G_TYPE_STRING, EPHY_NODE_BMK_PROP_LOCATION,
-				   0, NULL, &(editor->priv->address_col));
+				   EPHY_NODE_VIEW_SORTABLE |
+				   EPHY_NODE_VIEW_ELLIPSIZED,
+				   NULL, &(editor->priv->address_col));
+	gtk_tree_view_column_set_min_width (editor->priv->address_col, 300);
+	gtk_tree_view_column_set_resizable (editor->priv->address_col, TRUE);
+
 	ephy_node_view_enable_drag_source (EPHY_NODE_VIEW (bm_view),
 					   bmk_drag_types,
 					   G_N_ELEMENTS (bmk_drag_types),
@@ -1914,6 +1812,27 @@ ephy_bookmarks_editor_construct (EphyBookmarksEditor *editor)
 	gtk_container_add (GTK_CONTAINER (scrolled_window), bm_view);
 	gtk_widget_show (bm_view);
 	editor->priv->bm_view = bm_view;
+
+	action = gtk_action_group_get_action (action_group, "ViewTitle");
+	g_settings_bind (EPHY_SETTINGS_STATE,
+			 EPHY_PREFS_STATE_BOOKMARKS_VIEW_TITLE,
+			 action, "active",
+			 G_SETTINGS_BIND_DEFAULT);
+	g_settings_bind (EPHY_SETTINGS_STATE,
+			 EPHY_PREFS_STATE_BOOKMARKS_VIEW_TITLE,
+			 editor->priv->title_col, "visible",
+			 G_SETTINGS_BIND_DEFAULT);
+
+	action = gtk_action_group_get_action (action_group, "ViewAddress");
+	g_settings_bind (EPHY_SETTINGS_STATE,
+			 EPHY_PREFS_STATE_BOOKMARKS_VIEW_ADDRESS,
+			 action, "active",
+			 G_SETTINGS_BIND_DEFAULT);
+	g_settings_bind (EPHY_SETTINGS_STATE,
+			 EPHY_PREFS_STATE_BOOKMARKS_VIEW_ADDRESS,
+			 editor->priv->address_col, "visible",
+			 G_SETTINGS_BIND_DEFAULT);
+
 	g_signal_connect (G_OBJECT (bm_view),
 			  "key_press_event",
 			  G_CALLBACK (key_pressed_cb),
@@ -1922,6 +1841,10 @@ ephy_bookmarks_editor_construct (EphyBookmarksEditor *editor)
 			  "node_activated",
 			  G_CALLBACK (ephy_bookmarks_editor_node_activated_cb),
 			  editor);
+	g_signal_connect (G_OBJECT (bm_view),
+			  "node_middle_clicked",
+			  G_CALLBACK (ephy_bookmarks_editor_node_middle_clicked_cb),
+			  editor);
 	g_signal_connect (G_OBJECT (bm_view),
 			  "popup_menu",
 			  G_CALLBACK (ephy_bookmarks_editor_show_popup_cb),
@@ -1940,12 +1863,12 @@ ephy_bookmarks_editor_construct (EphyBookmarksEditor *editor)
 			       "bookmarks_paned",
 			       130);
 
-	set_columns_visibility (editor, details_value);
-
 	/* Lockdown settings */
 	action = gtk_action_group_get_action (action_group, "Export");
 	gtk_action_set_sensitive (action,
-				  eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_SAVE_TO_DISK) == FALSE);
+				  g_settings_get_boolean
+				   (EPHY_SETTINGS_LOCKDOWN,
+				    EPHY_PREFS_LOCKDOWN_SAVE_TO_DISK) == FALSE);
 }
 
 void
